#define RED_PIN   PD5
#define GREEN_PIN PD6
#define BLUE_PIN  PD7

#define MAGNET_PIN PB5

#define LCD_CMD_DISPLAY_OFF_CURSOR_OFF 0x0C
 
#include <avr/io.h>
#include <avr/interrupt.h> // Pre fungovanie random genertora
#include <stdlib.h> // Pre funkcie rand() a srand()
#include <stdio.h>
#include "uart.h"
#include "lcd.h"

FILE mystdout = FDEV_SETUP_STREAM(uart_putc, NULL, _FDEV_SETUP_WRITE);
    
static volatile int randomNumber = 0;
static volatile int priznak = 0;
static volatile int randomNum = 0;

int generateRandomNumber(int min, int max) { // Zadefinovanie funkcie genertora nhodch sel.
    randomNum = rand() % (max - min + 1);
    randomNum += min;
    return randomNum;
}

ISR(TIMER1_OVF_vect) {
    TCNT1 = 65536 - 45000; // Nastavenie timera na 5 seknd.
	priznak = 1;
}

void stavy() {
	if (randomNum == 1) {	  
		lcd_command(0x80); // Nastavenie pozcie kurzora na 1. riadok, 1. polko.
		lcd_puts("Teplota: 30\x01\x02\nJasno, Bez vetra");
		setRGBled(255, 0, 0);
	}
	  
	if (randomNum == 2) { 
		lcd_command(0x80);
		lcd_puts("Teplota: 25\x01\x02\nSlne\x04no, Veterno");
		setRGBled(255, 165, 0);
	}
	  
	if (randomNum == 3) {
		lcd_command(0x80);
		lcd_puts("Teplota: 16\x01\x02\nJasno, Slne\x04no  ");
		setRGBled(255, 255, 0);
	}
	  
	if (randomNum == 4) {
		lcd_command(0x80);
		lcd_puts("Teplota: 8\x01\x02 \nPoloobla\x04no,D\x03\x05\x06");
		setRGBled(102, 204, 255);
	}
	  
	if (randomNum == 5) {
		lcd_command(0x80);
		lcd_puts("Teplota: 2\x01\x02 \nZamra\x04\x07n\x08, D\x03\x05\x06 ");
		setRGBled(153, 0, 204);
	}
	  
	if (randomNum == 6) {
		lcd_command(0x80);
		lcd_puts("Teplota: -5\x01\x02\nZamra\x04\x07n\x08, Sneh ");
		setRGBled(0, 0, 255);
	}
}

int main(void) {
  uart_init(); // Inicializcia UARTu
  initRGBled(); // Inicilizcia RGB LED didy
  stdout = &mystdout; // printf() funkcia je zapnut

  printf("Spustam inicializaciu programu...\n");
  
  DDRB &= ~(1 << MAGNET_PIN); // Nastavenie MAGNET pinu na INPUT
  PORTB |= (1 << MAGNET_PIN);
  
  TCCR1B |= (1 << CS12) | (1 << CS10); // Prescaler nastaven na 1024
  TIMSK1 |= (1 << TOIE1);
  
  TCCR0B |= (1 << CS00);
  
  DDRD |= (1<<LCD_EN_pin);	// Pin D4 (Enable)  PORTD  output  
  DDRD |= (1<<LCD_RW_pin);	// Pin D3 (RW)      PORTD  output  
  DDRD |= (1<<LCD_RS_pin);	// Pin D2 (RS)      PORTD  output   
  
  
  LCD_DATA_PORT |= (1<<LCD_D4_pin)|(1<<LCD_D5_pin)|(1<<LCD_D6_pin)|(1<<LCD_D7_pin);	// Piny 1,2,3,4, PORTB ako output (Data pre display)
  
  lcd_init(); // Inicializcia LCD Displeja
  def_spec_znaky(); // Inicializcia pecilnych znakov na displeji
  lcd_command(LCD_CMD_DISPLAY_OFF_CURSOR_OFF); // Vypnutie kurzora na displeji
  
  printf("Inicializacia bola uspesna!\n");
  
  sei();
  
  while(1) { 
	  if (PINB & (1 << MAGNET_PIN)) {
			// Ak je magnet pripojen, do UARTu sa vype DEBUG, RGB LED dida sa vypne a vynuluje sa LCD Displej
			printf("Magnet je pripojeny!\r");
			setRGBled(0, 0, 0);
 			lcd_command(0x01);
        } else {
			// Ak je magnet odpojen, do UARTu sa vype DEBUG a spust sa funkcia stavy().
			printf("Magnet je odpojeny!\r");
			
			if (priznak == 1) {
				randomNum = generateRandomNumber(1, 6); // Nastavenie genertora nhodnch sel, aby generovalo len v intervale od 1 po 6.
				stavy();
				priznak = 0;
			}
		}
  }
  
  return(0);
}
